# The lines from here to the =cut are part of Perl's
# internal documentation system.  If you want 
# view the documentation use the command:
#
#	pod2text <script>
#
=pod

=head1 NAME

time_hash - Time handling functions (using hashs to store data)

=head1 SYNOPSIS

    use time_hash;

    $time1 = time_hash::new($hours, $minutes);

    $time3 = time_hash::add($time1, $time2);

    $time3 = time_hash::subtract($time1, $time2);

    $str = time_hash::to_string($time1);

=head1 DESCRIPTION

The I<time_hash> function provides you with a package that
adds and subtracts time as well as turns times into a string.

Note: This is a demonstration package, so several things that you should
do in a normal package were not done.  (See the BUGS section.)

=head1 FUNCTIONS

=over 4

=item I<$time1> = B<time_hash::new(>I<$hours, $minutes>B<);>

Create a new time hash.  The internals of the hash do not concern you,
the values in the hash are manipulated by the other I<time_hash> functions.

=item I<$time3> = B<time_hash::add(>I<$time1, $time2>B<);>

Adds I<$time1> += I<$time2> and return a reference to I<$time1>.

=item I<$time3> = B<time_hash::subtract(>I<$time1, $time2>B<);>

Subtract I<$time1> -= I<$time2> and return a reference to I<$time1>.

=item I<$str> = B<time_hash::to_string(>I<$time1>B<);>

Turns a time has into a string.

=back

=head1 BUGS

This is a demosntration package.  As such it does not export
the functions it provides.  This is a lousy way to write a package,
but a good way of making a point in a book.

The functions do a poor job of limit checking.  
Time differences bigger than 24 are not handled.  Input is not
checked in B<new>, so it will think that a time of 1:70 is leagl.

=head1 AUTHOR

Steve Oualline, E<lt>oualline@www.oualline.comE<gt>.

=head1 COPYRIGHT

Copyright 2002 Steve Oualline.
This program is distributed under the GPL.  

=cut
use strict;
use warnings;
#
# A package to handle time data (implement using a hash)
#
package time_hash;

################################################################
# $time = $new($hours, $minutes)
#
# Create a new time hash and return a reference to it.
################################################################
sub new($$)
{
    my $hours = shift; # Initial hours
    my $minutes = shift; # Initial minutes
    my $self = {};	# The hash we are creating

    $self->{hours} = $hours;
    $self->{minutes} = $minutes;
    return ($self);
}

################################################################
# $result = add($time1, $time2)
#
# $time1 += $time2
#
# Return a reference to time1
################################################################
sub add($$)
{
    my $self = shift;	# The hash we wish to add to
    my $other = shift;	# The other hash

    $self->{hours} += $other->{hours};
    $self->{minutes} += $other->{minutes};
    $self->{hours} += int($self->{minutes} / 60);
    $self->{minutes} %= 60;
}

################################################################
# $result = subtract($time1, $time2)
#
# $time1 -= $time2
#
# Return a reference to time1
################################################################
sub subtract($$)
{
    my $self = shift;	# The hash we wish to subtract from
    my $other = shift;	# The other hash

    $self->{hours} -= $other->{hours};
    $self->{minutes} -= $other->{minutes};
    while ($self->{minutes} < 0) {
        $self->{minutes} += 60;
        $self->{hours}--;
    }
}

################################################################
# $str = to_string($time)
#
# Turn a time into a string.
################################################################
sub to_string($)
{
    my $self = shift;	# The hash to turn into a string

    return(sprintf("%d:%02d", 
	$self->{hours}, $self->{minutes}));
}

1;
