=pod

=head1 NAME

web.pm - Analyze log files.

=head1 SYNOPSIS

    use web;
    my @hit_info; 	
    read_info($file_name)
    $info = do_count()		

=head1 DESCRIPTION

The I<web.pm> module provides function to read and analyze
Apache web logs.

=head1 Exports

=over 4

=item @hit_info

An array of hashes containing information about each web hit.
The fields in each entry is:

=over 4

=item B<host>

The host which read this page

=item B<date>

The date/time when it was accessed.

=item B<page>

The page.

=back

=item read_file($file_name)

Read a Apache log file and store the results in I<@hit_info>.

=item $result = do_count()

Return an array of hashes contains a compilation of the data
in I<@hit_info>.  The result is an array of hashes.  The fields
in the hash are:  page -- The page name, count -- Number of time it
was accessed.

=back

=cut
package web;
use strict;
use warnings;

require Exporter;
use vars qw(@EXPORT @ISA @hit_info);
@ISA = 'Exporter';
@EXPORT = qw(&read_info @hit_info &do_count);

@hit_info = ();

##################################################
# read_info($file_name) -- Read information about
#	hits.
##################################################
sub read_info($)
{
    my $file_name = shift;	# Get the name of the file

    open IN_FILE, "<$file_name" or 
	die("Could not open $file_name");

    while (<IN_FILE>) {
	#     ++++++------------------------------------------ Digits or dot
	#     ||||||+----------------------------------------- Zero or more
	#    +|||||||+---------------------------------------- Put in $1
	#    |||||||||++-------------------------------------- Space
	#    |||||||||||+------------------------------------- Zero or more
	#    ||||||||||||+------------------------------------ Char -
	#    |||||||||||||++---------------------------------- Space
	#    |||||||||||||||+--------------------------------- Zero or more
	#    ||||||||||||||||+-------------------------------- Char -
	#    |||||||||||||||||++------------------------------ Space
	#    |||||||||||||||||||+----------------------------- Zero or more
	#    ||||||||||||||||||||++--------------------------- Char [
	#    |||||||||||||||||||||| +++++--------------------- Anything except ]
	#    |||||||||||||||||||||| |||||+-------------------- Zero or more
	#    ||||||||||||||||||||||+||||||+------------------- Put in $2
	#    ||||||||||||||||||||||||||||||++----------------- ] Char.
	#    ||||||||||||||||||||||||||||||||+---------------- Space Char.
	#    |||||||||||||||||||||||||||||||||+--------------- " Char.
	#    |||||||||||||||||||||||||||||||||| ++------------ Non-space
	#    |||||||||||||||||||||||||||||||||| ||+----------- One or more
	#    ||||||||||||||||||||||||||||||||||+|||+---------- Put in $3
	#    |||||||||||||||||||||||||||||||||||||||++-------- Space
	#    |||||||||||||||||||||||||||||||||||||||||+------- One or more
	#    |||||||||||||||||||||||||||||||||||||||||| ++---- Non-space
	#    |||||||||||||||||||||||||||||||||||||||||| ||+--- One or more
	#    ||||||||||||||||||||||||||||||||||||||||||+|||+-- Put in $4
	if (/([0-9.]*)\s*-\s*-\s*\[([^\]]*)\] "(\S+)\s+(\S+)/) {
	    # $3 -- The operation code
	    if ($3 ne "GET") {
		next;
	    }
	    # Entry we are adding
	    my %new_entry = (
		    host => $1,
		    date => $2,
		    page => $4
			    );
	    push(@hit_info, {%new_entry});
	}
    }
    close IN_FILE;
}


#######################################################
# do_count -- Count the number of hits in the data.
#
# Returns an array reference.  Each element is a hash
# with the following keys:
#	page -- The page name
#	count -- Number of element in the page
#######################################################
sub do_count()
{
    # Count of the number of hits
    # 	key = page
    #	value = count
    my %counts = ();
    foreach my $cur_hit (@hit_info) {
	if (not defined($counts{$cur_hit->{page}})) {
	    $counts{$cur_hit->{page}} = 1;
	} else {
	    $counts{$cur_hit->{page}}++;
	}
    }

    my @stat_array = ();	# Array of statistics

    # Loop through the entire count array
    foreach my $cur_count (sort keys %counts) {
	push(@stat_array, {
		page => $cur_count,
		count => $counts{$cur_count}
		});
    }
    return (\@stat_array);
}

1;
